package com.itko.lisa.ext.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import com.google.common.io.ByteStreams;
import com.itko.lisa.model.mar.ModelArchive;

/**
 * MarHelper - <Nice descriptive name of class>
 *
 * @author <a href="mailto:daniel.bingham@itko.com">Daniel Bingham</a>
 */
public class MarHelper
{
    private static final Logger logger = Logger.getLogger( MarHelper.class.getName() );

    public static ModelArchive addAndreplaceFilesInMar( final ModelArchive mar,
                                                        final File[] replacements )
    {
        try
        {
            byte[] marBytes = mar.getBytes();

            ZipInputStream zin = new ZipInputStream( new ByteArrayInputStream( marBytes ) );
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            ZipOutputStream zout = new ZipOutputStream( baos );

            byte[] buf = new byte[1024];

            ZipEntry entry = zin.getNextEntry();
            while( entry != null )
            {
                String name = entry.getName();
                boolean notNew = true;
                for( File f : replacements )
                {
                    if( f.getName().equals( name ) )
                    {
                        notNew = false;
                        break;
                    }
                }
                if( notNew )
                {
                    zout.putNextEntry( new ZipEntry( name ) );
                    ByteStreams.copy( zin, zout );
                }
                entry = zin.getNextEntry();
            }
            zin.close();
            for( File replacement : replacements )
            {
                InputStream in = new FileInputStream( replacement );
                zout.putNextEntry( new ZipEntry( replacement.getName() ) );
                ByteStreams.copy( in, zout );
                zout.closeEntry();
                in.close();
            }
            // Complete the ZIP file
            zout.close();

            byte[] updated = baos.toByteArray();

            return new ModelArchive( updated );
        }
        catch( Exception e )
        {
            logger.log( Level.SEVERE, e.getMessage(), e );
            throw new RuntimeException( e.getMessage(), e );
        }
    }
}
