package com.itko.lisa.ext.util;

import java.sql.ResultSet;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.itko.lisa.test.TestExec;
import com.itko.lisa.vse.stateful.model.Request;
import com.itko.lisa.vse.stateful.model.Response;
import com.itko.lisa.vse.stateful.model.Streamer;
import com.itko.lisa.vse.stateful.protocol.http.HttpProtocolHandler;
import com.itko.lisa.vse.stateful.protocol.jdbc.JDBCProtocolHandler;

/**
 * Invoke a LISA test through the lisa-invoke web app.
 * 
 * @author <a href="mailto:luther@itko.com">Luther E. Birdzell</a>
 */
public class VseTransactionHelper
{

    private static Logger      logger                     = Logger.getLogger(VseTransactionHelper.class.getName());

    /**
     * 
     */
    public static final String PROPERTY_LISA_VSE_RESPONSE = "lisa.vse.response";
    /**
     * 
     */
    public static final String PROPERTY_LISA_VSE_REQUEST  = "lisa.vse.request";

    /**
     * 
     * 
     * @param testExec
     * @return
     */
    public static Request getRequest(TestExec testExec)
    {
        Request request = null;

        // 5.0
        Object requestObj = testExec.getStateValue(PROPERTY_LISA_VSE_REQUEST);

        // JDBC default 4.x
        if ( requestObj == null )
        {
            requestObj = testExec.getStateValue("lisa.JDBC Listen.rsp");
        }

        // HTTP default 4.x
        if ( requestObj == null )
        {
            requestObj = testExec.getStateValue("lisa.HTTP Listen.rsp");
        }

        // last resort
        if ( requestObj == null )
        {
            requestObj = testExec.getLastResponse();
        }

        if ( requestObj instanceof String )
        {
            Object tmpReq = Streamer.fromXML( (String) requestObj );
            if(tmpReq instanceof Request)
            {
                request = (Request) tmpReq;
            }
        }
        else if ( requestObj instanceof Request )
        {
            request = ( Request ) requestObj;
        }

        return request;
    }

    /**
     * 
     * 
     * @param responseBody
     * @return
     */
    public static String getHttpResponse(String responseBody)
    {
        try
        {
            Response r = new Response();

            // Set the response headers here. These two are required.
            r.getMetaData()
             .put(HttpProtocolHandler.RESPONSE_CODE_HEADER, "200");

            r.getMetaData().put(HttpProtocolHandler.RESPONSE_CODE_HEADER, "OK");

            r.getMetaData().put("Content-Type", "text/xml");

            // TODO expand the filter to support this
            // ... followed by any other HTTP response headers you’ll need.

            // Note that Content-Length is handled automatically.

            // Now set the body for the response...
            r.setBody(responseBody);

            // Use this most of the time...
            // r.createWorkingCopy();

            // Use this if you want XML to play with further; both are
            // acceptable to the respond step.
            return Streamer.toXML( r.createTransientCopy() );
        }
        catch ( Exception e )
        {
            logger.log(Level.SEVERE, e.getMessage(), e);
            throw new RuntimeException(e.getMessage(), e);
        }
    }

    /**
     * 
     * 
     * @param rs
     * @return
     */
    public static String getJdbcResponse(ResultSet rs)
    {
        try
        {
            Response response = JDBCProtocolHandler.createQueryResponse(rs);
            return Streamer.toXML( response.createTransientCopy() );
        }
        catch ( Exception e )
        {
            logger.log(Level.SEVERE, e.getMessage(), e);
            throw new RuntimeException(e.getMessage(), e);
        }
    }
}
