/*
 * Copyright 2006 Luther E. Birdzell.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package com.itko.lisa.ext.util;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.xerces.parsers.DOMParser;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Utility methods for working with XML.
 * 
 * @author <a href="mailto:luther@dctmutils.org">Luther E. Birdzell</a>
 */
public class XmlHelper
{
    private static Logger log = Logger.getLogger(XmlHelper.class.getName());

    /**
     * All methods are static.
     */
    protected XmlHelper()
    {
    }

    /**
     * 
     * 
     * @param xsl
     * @param source
     * @return the transformed xml
     */
    public static String transform(String xsl, String source)
    {
        String output = null;

        try
        {
            // LB - 29 July 2008
            // JDOM was having trouble when transforming XML in to unstructured
            // forms like a text file so we switched

            // SAXBuilder builder = new SAXBuilder();
            // Document sourceDoc = builder.build(new StringReader(source));
            // Document xslDoc = builder.build(new StringReader(xsl));
            //
            // XSLTransformer transformer = new XSLTransformer(xslDoc);
            //
            // Document outputDoc = transformer.transform(sourceDoc);
            // XMLOutputter outputter = new XMLOutputter();
            // outputter.setFormat(Format.getPrettyFormat());
            // output = outputter.outputString(outputDoc);

            Transformer transform = TransformerFactory.newInstance()
                                                      .newTransformer(new StreamSource(new StringReader(xsl)));
            final StringWriter writer = new StringWriter();
            transform.transform(new StreamSource(new StringReader(source)),
                                new StreamResult(writer));

            output = writer.toString();
        }
        catch ( Exception e )
        {
            log.log(Level.SEVERE, e.getMessage(), e);
            throw new RuntimeException(e.getMessage(), e);
        }

        return output;
    }

    /**
     * Convert a jdom <code>Document</code> into a <code>String</code>.
     * 
     * @param doc
     * @return an XML <code>String</code>
     */
    public static String toString(Document doc)
    {
        String xml = null;

        XMLOutputter outputter = new XMLOutputter();
        outputter.setFormat(Format.getPrettyFormat());
        xml = outputter.outputString(doc);
        return xml;
    }

    /**
     * Convert a jdom <code>Element</code> into a <code>String</code>.
     * 
     * @param element
     * @return an XML <code>String</code>
     */
    public static String toString(Element element)
    {
        String xml = null;

        XMLOutputter outputter = new XMLOutputter();
        outputter.setFormat(Format.getPrettyFormat());
        xml = outputter.outputString(element);
        return xml;
    }

    /**
     * 
     * 
     * @param requestXml
     * @return
     * @throws SAXException
     * @throws IOException
     */
    public static String normalize(String xml) throws SAXException, IOException
    {
        if ( xml == null )
        {
            return xml;
        }

        DOMParser parser = new DOMParser();
        ByteArrayInputStream bais = new ByteArrayInputStream(xml.getBytes());
        parser.parse(new InputSource(bais));
        org.w3c.dom.Document doc = parser.getDocument();
        doc.normalizeDocument();

        return toString(doc);
    }

    /**
     * toString() method for XML Nodes.
     * 
     * @param doc
     * @return the <code>Node</code> as a <code>String</code>
     */
    public static String toString(org.w3c.dom.Document doc)
    {
        if ( doc == null )
        {
            return null;
        }

        StringBuffer stringBuffer = null;

        try
        {
            Transformer transformer = TransformerFactory.newInstance()
                                                        .newTransformer();

            StringWriter stringWriter = new StringWriter();

            transformer.transform(new DOMSource(doc),
                                  new StreamResult(stringWriter));

            stringBuffer = stringWriter.getBuffer();

            return stringBuffer.toString();
        }
        catch ( TransformerException te )
        {
            te.printStackTrace();
            throw new RuntimeException(te.getMessage(), te);
        }
    }

}
